<?php

namespace Drupal\entity_reference_patterns\Element;

use Drupal\Core\Entity\Element\EntityAutocomplete as EntityAutocompleteCore;
use Drupal\Component\Utility\Tags;
use Drupal\entity_reference_patterns\Entity\EntityReferencePatternEntity;

/**
 * Extends core FormElement("entity_autocomplete") to support custom default value labels.
 */
class EntityAutocomplete extends EntityAutocompleteCore {

  /**
   * {@inheritdoc}
   */
  public function getInfo() {
    $info = parent::getInfo();
    $info['#attached']['library'][] = 'entity_reference_patterns/entity_reference_patterns.autocomplete';
    return $info;
  }

  /**
   * {@inheritdoc}
   */
  public static function getEntityLabels(array $entities) {
    /** @var \Drupal\Core\Entity\EntityRepositoryInterface $entity_repository */
    $entity_repository = \Drupal::service('entity.repository');
    $token = \Drupal::token();

    $entity_labels = [];
    foreach ($entities as $entity) {
      // Set the entity in the correct language for display.
      $entity = $entity_repository->getTranslationFromContext($entity);
      $entity_ids = \Drupal::entityQuery('entity_reference_pattern')
        ->condition('type', $entity->getEntityType()->id())
        ->condition('status', 1)
        ->sort('weight', 'DESC')
        ->range(0, 1)
        ->execute();
      if ($labels = EntityReferencePatternEntity::loadMultiple($entity_ids)) {
        $label_config_entity = reset($labels);
        $criteria = $label_config_entity->getSelectionCriteria();

        if (!empty($criteria['bundles'])) {
          foreach ($criteria as $type => $condition) {
            if ($type == 'bundles') {
              foreach ($condition as $k => $v) {
                if ($k == $entity->bundle()) {
                  $label = ($entity->access('view label'))
                    ? $token->replace($label_config_entity->getPattern(), [$entity->getEntityType()->id() => $entity], ['clear' => TRUE])
                    : t('- Restricted access -');
                }
              }
            }
          }
        }
        else {
          $label = ($entity->access('view label'))
            ? $token->replace($label_config_entity->getPattern(), [$entity->getEntityType()->id() => $entity], ['clear' => TRUE])
            : t('- Restricted access -');
        }
        if (empty($label)) {
          $label = ($entity->access('view label')) ? $entity->label() : t('- Restricted access -');
        }
        // Just to be safe here we replace ([digital]) petterns so referenced ID could be extracted.
        $label = preg_replace("/\(([\d\h,]+)\)/", "[$1]", $label);
        // Take into account "autocreated" entities.
        if (!$entity->isNew()) {
          $label .= ' (' . $entity->id() . ')';
        }
      }
      else {
        // Use the special view label, since some entities allow the label to be
        // viewed, even if the entity is not allowed to be viewed.
        $label = ($entity->access('view label')) ? $entity->label() : t('- Restricted access -');
        // Take into account "autocreated" entities.
        if (!$entity->isNew()) {
          $label .= ' (' . $entity->id() . ')';
        }
      }
      // Labels containing commas or quotes must be wrapped in quotes.
      $entity_labels[] = Tags::encode($label);
    }

    return implode(', ', $entity_labels);
  }

}
