<?php

namespace Drupal\entity_reference_patterns\Entity;

use Drupal\Core\Config\Entity\ConfigEntityBase;

/**
 * Defines the Entity Reference Patterns entity.
 *
 * @ConfigEntityType(
 *   id = "entity_reference_pattern",
 *   label = @Translation("Entity Reference Patterns"),
 *   handlers = {
 *     "list_builder" = "Drupal\entity_reference_patterns\Controller\EntityReferencePatternListBuilder",
 *     "form" = {
 *       "default" = "Drupal\entity_reference_patterns\Entity\Form\PatternEditForm",
 *       "edit" = "Drupal\entity_reference_patterns\Entity\Form\PatternEditForm",
 *       "duplicate" = "Drupal\entity_reference_patterns\Entity\Form\PatternDuplicateForm",
 *       "delete" = "Drupal\Core\Entity\EntityDeleteForm",
 *     },
 *     "route_provider" = {
 *       "html" = "Drupal\Core\Entity\Routing\DefaultHtmlRouteProvider",
 *     },
 *     "access" = "Drupal\entity_reference_patterns\Entity\AccessControlHandler\AccessControlHandler",
 *   },
 *   config_prefix = "pattern",
 *   admin_permission = "administer entity reference pattern",
 *   entity_keys = {
 *     "id" = "id",
 *     "label" = "label",
 *     "uuid" = "uuid",
 *     "weight" = "weight",
 *     "status" = "status"
 *   },
 *   config_export = {
 *     "id",
 *     "label",
 *     "type",
 *     "pattern",
 *     "selection_criteria",
 *     "weight"
 *   },
 *   lookup_keys = {
 *     "type",
 *     "status",
 *   },
 *   links = {
 *     "collection" = "/admin/config/search/entity-reference-patterns",
 *     "duplicate" = "/admin/config/search/entity-reference-patterns/{entity_reference_pattern}/duplicate",
 *     "new" = "/admin/config/search/entity-reference-patterns/new",
 *     "edit" = "/admin/config/search/entity-reference-patterns/{entity_reference_pattern}/edit",
 *     "delete" = "/admin/config/search/entity-reference-patterns/{entity_reference_pattern}/delete",
 *   }
 * )
 */
class EntityReferencePatternEntity extends ConfigEntityBase implements EntityReferencePatternEntityInterface {

  /**
   * The name (plugin ID) of the tour.
   *
   * @var string
   */
  protected $id;

  /**
   * The module which this Entity Reference Pattern is assigned to.
   *
   * @var string
   */
  protected $module;

  /**
   * The label of the Entity Reference Pattern.
   *
   * @var string
   */
  protected $label;

  /**
   * The pattern type.
   *
   * @var string
   */
  protected $type;

  /**
   * A tokenized string for entity reference pattern generation.
   *
   * @var string
   */
  protected $pattern;

  /**
   * The plugin configuration for the selection criteria condition plugins.
   *
   * @var array
   */
  protected $selection_criteria = [];

  /**
   * @var int
   */
  protected $weight = 0;

  /**
   * {@inheritdoc}
   */
  public function getPattern() {
    return $this->pattern;
  }

  /**
   * {@inheritdoc}
   */
  public function setPattern($pattern) {
    $this->pattern = $pattern;
    return $this;
  }

  /**
   * {@inheritdoc}
   */
  public function getType() {
    return $this->type;
  }

  /**
   * {@inheritdoc}
   */
  public function setType($type) {
    $this->type = $type;
    return $this;
  }

  /**
   * {@inheritdoc}
   */
  public function getModule() {
    return $this->module;
  }

  /**
   * {@inheritdoc}
   */
  public function setModule($module) {
    $this->module = $module;
    return $this;
  }

  /**
   * {@inheritdoc}
   */
  public function getSelectionCriteria() {
    if (empty(array_values($this->selection_criteria))) {
      return FALSE;
    }
    else {
      return $this->selection_criteria;
    }
  }

  /**
   * {@inheritdoc}
   */
  public function setSelectionCriteria(array $selection_criteria) {
    $this->selection_criteria = $selection_criteria;
    return $this;
  }

  /**
   * {@inheritdoc}
   */
  public function getWeight() {
    return $this->weight;
  }

  /**
   * {@inheritdoc}
   */
  public function setWeight($weight) {
    $this->weight = $weight;
    return $this;
  }

  /**
   * Get entity type label.
   */
  public function getEntityTypeLabel() {
    // Get all applicable entity types.
    foreach (\Drupal::entityTypeManager()->getDefinitions() as $entity_type_id => $entity_type) {
      if ($this->getType() == $entity_type_id) {
        return $entity_type->getLabel();
      }
    }
  }

}
