<?php

namespace Drupal\entity_reference_patterns;

use Drupal\Core\Entity\EntityRepositoryInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Utility\Token;
use Drupal\entity_reference_patterns\Entity\EntityReferencePatternEntity;

/**
 * Entity Reference Patterns module service.
 *
 * @ingroup entity_reference_patterns
 */
class EntityReferencePatterns {

  /**
   * The entity type manager service.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * The entity repository.
   *
   * @var \Drupal\Core\Entity\EntityRepositoryInterface
   */
  protected $entityRepository;

  /**
   * The token manager.
   *
   * @var \Drupal\Core\Utility\Token
   */
  protected $token;

  /**
   * @param EntityTypeManagerInterface $entity_type_manager
   * @param EntityRepositoryInterface $entity_repository
   * @param Token $token
   */
  public function __construct(EntityTypeManagerInterface $entity_type_manager, EntityRepositoryInterface $entity_repository, Token $token) {
    $this->entityTypeManager = $entity_type_manager;
    $this->entityRepository = $entity_repository;
    $this->token = $token;
  }

  /**
   * Implements hook_options_list_alter().
   */
  public function hook_options_list_alter(&$options, $context) {
    $fieldDefinition = $context['fieldDefinition']->getFieldStorageDefinition();
    $settings = $fieldDefinition->getSettings();
    $new_options = [];
    if (!empty($settings['target_type'])) {
      $target_type = $settings['target_type'];
      foreach ($options as $bundle_label => $items) {
        if ($bundle_label !== '_none' && is_array($items)) {
          foreach ($items as $entity_id => $label) {
            $entity = $this->entityTypeManager->getStorage($target_type)->load($entity_id);
            // Set the entity in the correct language for display.
            $entity = $this->entityRepository->getTranslationFromContext($entity);
            $entity_ids = $this->entityTypeManager->getStorage('entity_reference_pattern')->getQuery()
              ->accessCheck(TRUE)
              ->condition('type', $entity->getEntityType()->id())
              ->condition('status', 1)
              ->sort('weight', 'DESC')
              ->range(0, 1)
              ->execute();
            if ($labels = EntityReferencePatternEntity::loadMultiple($entity_ids)) {
              $label_config_entity = reset($labels);
              $criteria = $label_config_entity->getSelectionCriteria();

              if (!empty($criteria['bundles'])) {
                foreach ($criteria as $type => $condition) {
                  if ($type == 'bundles') {
                    foreach ($condition as $k => $v) {
                      if ($k == $entity->bundle()) {
                        $new_label = $this->token->replace(
                          $label_config_entity->getPattern(),
                          [$entity->getEntityType()->id() => $entity],
                          ['clear' => TRUE]
                        );
                      }
                    }
                  }
                }
              }
              else {
                $new_label = $this->token->replace(
                  $label_config_entity->getPattern(),
                  [$entity->getEntityType()->id() => $entity],
                  ['clear' => TRUE]
                );
              }
              $new_options[$bundle_label][$entity_id] = isset($new_label) ? $new_label : $label;
            }
            else {
              $new_options[$bundle_label][$entity_id] = $label;
            }
          }
        }
        else {
          $new_options[$bundle_label] = $items;
        }
      }
    }
    if (!empty($new_options)) {
      $options = $new_options;
    }
  }

}
